#Get-IOPS v1 
#11/8/2018
#Script to get VM storage info
#By Tyler Verkade

#Define metrics and interval information
$metrics = "disk.numberwrite.summation","disk.numberread.summation"
$start = (Get-Date).AddMinutes(-60)
$report = @()

#Define source machine(s) or resource pools from which to pull the data
$vms = Get-VM | where {$_.PowerState -eq "PoweredOn"}
$stats = Get-Stat -Realtime -Stat $metrics -Entity $vms -Start $start
$interval = $stats[0].IntervalSecs
$storageprov = @{}
$storageused = @{}
$ResourcePool = @{}

#Sub to pull LUN info for each disk on each VM
$lunTab = @{}
foreach($ds in (Get-Datastore -VM $vms | where {$_.Type -eq "VMFS"})){
	$ds.ExtensionData.Info.Vmfs.Extent | %{
		$lunTab[$_.DiskName] = $ds.Name
	}
}

#Define the report parameters
$report = $stats | Group-Object -Property {$_.Entity.Name},Instance | %{
	New-Object PSObject -Property @{
		VM = $_.Values[0]
 		Disk = $_.Values[1]
 		IOPSMax = ($_.Group | `
 			Group-Object -Property Timestamp | `
 			%{$_.Group[0].Value + $_.Group[1].Value} | `
 			Measure-Object -Maximum).Maximum / $interval
		Datastore = $lunTab[$_.Values[1]]
		StorageUsed = (Get-VM $_.Values[0] | Select UsedSpaceGB).UsedSpaceGB
		StorageProv = (Get-VM $_.Values[0] | Select ProvisionedSpaceGB).ProvisionedSpaceGB
		ResourcePool = (Get-VM $_.Values[0] | Select ResourcePool).ResourcePool
	}
}

#Display the report
$report