#Begin
Import-Module ImportExcel

#Global Variables
	#UCS Domains
		$kwd1 = "UCS-gdrsmiuk.kwd.vdc.local"			#172.19.1.20	
		$kwd2 = "UCS-gdrsmiuk-02.kwd.vdc.local"			#172.19.6.20	
		$kwd3 = "UCS-gdrsmiuk-03.kwd.vdc.local"			#172.19.6.10	
		$dtw1 = "UCS-MLRSMI00.dtw.vdc.local"			#172.16.50.60
		$dtw2 = "UCS-MLRSMI00-02.dtw.vdc.local"			#172.16.54.60
		$dtw3 = "UCS-MLRSMI00-03.dtw.vdc.local"			#172.16.54.70
		$ucsdomains = $kwd1,$kwd2,$kwd3,$dtw1,$dtw2,$dtw3
		
	#Credentials
		$c = Get-Credential

	#Misc Names
		$date = get-date -format yyyy-MM-dd

foreach($UCSDOMAIN in $UCSDOMAINS){

#Connect and Collect
Connect-Ucs $UCSDOMAIN -Credential $c
$filename = ".\" + "$Date_" + "$UCSDOMAIN" + "_Report.xlsx"
$UCSchassis = (get-ucschassis | select Id).ID

#Function to get UCS licensing
Get-UcsLicense | where {($_.Sku -eq 'UCS-LIC-10GE')}| Select-Object Ucs,Scope,Feature,Sku,AbsQuant,DefQuant,UsedQuant,@{n="RemQuant";e={$_.AbsQuant-$_.UsedQuant}},Status,PeerStatus,OperState,GracePeriodUsed | export-excel $filename -worksheetname LicenseInfo  -startcolumn 2 -startrow 2 -AutoSize

#Function to get empty slots for the domain
Get-UcsFabricComputeSlotEp | Where-Object {($_.Presence -eq 'empty')} | Select UCS,ChassisID,SlotID | Sort-Object -Property ChassisId | export-excel $filename -worksheetname EmptySlots  -startcolumn 2 -startrow 2 -AutoSize

#Function to get power stats for the domain
Get-UcsPowerBudget | Where {($_.PsuCapacity -ne '0')} | Select UCS,Dn,CurrentPower,OperPeak,PsuCapacity | Sort-Object -Property Dn | export-excel $filename -worksheetname ChassisPowerInfo  -startcolumn 2 -startrow 2 -AutoSize

#Function to collect chassis population report
ForEach ($UCSchassisID in $UCSchassis){
	#Set up output file
	$worksheetname = $UCSchassisID
	echo "Chassis #" | export-excel $filename -worksheetname $worksheetname  -startcolumn 1 -startrow 1 -AutoSize
	echo $UCSchassisID | export-excel $filename -worksheetname $worksheetname  -startcolumn 2 -startrow 1  -AutoSize
	#odd slots
	echo "Profile Assigned" | export-excel $filename -worksheetname $worksheetname -startcolumn 2 -startrow 2
	echo "Slot 1" | export-excel $filename -worksheetname $worksheetname  -startcolumn 1 -startrow 3 -AutoSize
	echo "Slot 3" | export-excel $filename -worksheetname $worksheetname  -startcolumn 1 -startrow 4 -AutoSize
	echo "Slot 5" | export-excel $filename -worksheetname $worksheetname  -startcolumn 1 -startrow 5 -AutoSize
	echo "Slot 7" | export-excel $filename -worksheetname $worksheetname  -startcolumn 1 -startrow 6 -AutoSize
	#even slots
	echo "Profile Assigned" | export-excel $filename -worksheetname $worksheetname -startcolumn 5 -startrow 2 -AutoSize
	echo "Slot 2" | export-excel $filename -worksheetname $worksheetname  -startcolumn 4 -startrow 3 -AutoSize
	echo "Slot 4" | export-excel $filename -worksheetname $worksheetname  -startcolumn 4 -startrow 4 -AutoSize
	echo "Slot 6" | export-excel $filename -worksheetname $worksheetname  -startcolumn 4 -startrow 5 -AutoSize
	echo "Slot 8" | export-excel $filename -worksheetname $worksheetname  -startcolumn 4 -startrow 6 -AutoSize
	
	#Set up columns for individual blade information
	echo "Blade" | export-excel $filename -worksheetname $worksheetname  -startcolumn 1 -startrow 8 -AutoSize
	echo "Assigned Profile" | export-excel $filename -worksheetname $worksheetname  -startcolumn 2 -startrow 8 -AutoSize
	echo "User Label" | export-excel $filename -worksheetname $worksheetname  -startcolumn 3 -startrow 8 -AutoSize
	echo "Model" | export-excel $filename -worksheetname $worksheetname  -startcolumn 4 -startrow 8 -AutoSize
	echo "VID" | export-excel $filename -worksheetname $worksheetname  -startcolumn 5 -startrow 8 -AutoSize
	echo "Serial #" | export-excel $filename -worksheetname $worksheetname  -startcolumn 6 -startrow 8 -AutoSize
	echo "Association Status" | export-excel $filename -worksheetname $worksheetname  -startcolumn 7 -startrow 8 -AutoSize
	echo "Total RAM" | export-excel $filename -worksheetname $worksheetname  -startcolumn 8 -startrow 8 -AutoSize
	echo "CPUs Installed" | export-excel $filename -worksheetname $worksheetname  -startcolumn 9 -startrow 8 -AutoSize
	echo "CPU Cores" | export-excel $filename -worksheetname $worksheetname  -startcolumn 10 -startrow 8 -AutoSize
	echo "UCS Domain" | export-excel $filename -worksheetname $worksheetname  -startcolumn 11 -startrow 8 -AutoSize
	
	$OddBlade = "Blank"
	$EvenBladedn = "Blank"
	$BladeAssignedToDn = "blank"
	$BladeUsrLbl = "blank"
	$BladeModel = "blank"
	$BladeVid = "blank"
	$BladeSerial = "blank"
	$BladeAssociation = "blank"
	$BladeTotalMemory = "blank"
	$BladeNumOfCpus = "blank"
	$BladeNumOfCores = "blank"
	$BladeSlotID = 0
		
	$OddBlade = "Blank"
	$OddBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 1)}).AssignedTodn
	echo $OddBlade | export-excel $filename -worksheetname $worksheetname -startcolumn 2 -startrow 3 -Autosize
	$OddBlade = "Blank"
	$OddBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 3)}).AssignedTodn
	echo $OddBlade | export-excel $filename -worksheetname $worksheetname -startcolumn 2 -startrow 4 -Autosize
	$OddBlade = "Blank"
	$OddBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 5)}).AssignedTodn
	echo $OddBlade | export-excel $filename -worksheetname $worksheetname -startcolumn 2 -startrow 5 -Autosize
	$OddBlade = "Blank"
	$OddBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 7)}).AssignedTodn
	echo $OddBlade | export-excel $filename -worksheetname $worksheetname -startcolumn 2 -startrow 6 -Autosize
	
	
	$EvenBlade = "Blank"
	$EvenBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 2)}).AssignedTodn
	echo $EvenBlade | export-excel $filename -worksheetname $worksheetname -startcolumn 5 -startrow 3 -Autosize
	$EvenBlade = "Blank"
	$EvenBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 4)}).AssignedTodn
	echo $EvenBlade | export-excel $filename -worksheetname $worksheetname -startcolumn 5 -startrow 4 -Autosize
	$EvenBlade = "Blank"
	$EvenBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 6)}).AssignedTodn
	echo $EvenBlade | export-excel $filename -worksheetname $worksheetname -startcolumn 5 -startrow 5 -Autosize
	$EvenBlade = "Blank"
	$EvenBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 8)}).AssignedTodn
	echo $EvenBlade | export-excel $filename -worksheetname $worksheetname -startcolumn 5 -startrow 6 -Autosize

	
	#Grab Slot Information
	#if(get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $worksheetname) -and ($_.SlotID -eq 1 -or $_.SlotID -eq 3 -or $_.SlotID -eq 5 -or $_.SlotID -eq 7)})
	#	{
	#	$dn = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $worksheetname) -and ($_.SlotID -eq 1 -or $_.SlotID -eq 3 -or $_.SlotID -eq 5 -or $_.SlotID -eq 7)}.AssignedToDn)
	#	echo $dn | export-excel $filename -worksheetname $worksheetname -startcolumn 2 -startrow 3
	#	}
	
	#if($dn = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $worksheetname) -and ($_.SlotID -eq 2 -or $_.SlotID -eq 4 -or $_.SlotID -eq 6 -or $_.SlotID -eq 8)}).AssignedToDn) 
	#	{
	#	echo $dn | export-excel $filename -worksheetname $worksheetname -startcolumn 5 -startrow 3
	#	}
		
	#Grab Blade Information
	$BladeSlotID = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).SlotID
	echo $BladeSlotID | export-excel $filename -worksheetname $worksheetname -startcolumn 1 -startrow 9 -AutoSize
	
	$BladeAssignedToDn = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).AssignedToDn 
	echo $BladeAssignedToDn | export-excel $filename -worksheetname $worksheetname -startcolumn 2 -startrow 9 -AutoSize
	
	$BladeUsrLbl = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).UsrLbl 
	echo $BladeUsrLbl | export-excel $filename -worksheetname $worksheetname -startcolumn 3 -startrow 9 -AutoSize

	$BladeModel = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).Model 
	echo $BladeModel | export-excel $filename -worksheetname $worksheetname -startcolumn 4 -startrow 9 -AutoSize
	
	$BladeVID = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).VID 
	echo $BladeVID | export-excel $filename -worksheetname $worksheetname -startcolumn 5 -startrow 9	-AutoSize
	
	$BladeSerial = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).Serial 
	echo $BladeSerial | export-excel $filename -worksheetname $worksheetname -startcolumn 6 -startrow 9 -AutoSize
	
	$BladeAssociation = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).Association
	echo $BladeAssociation | export-excel $filename -worksheetname $worksheetname -startcolumn 7 -startrow 9 -AutoSize
	
	$BladeTotalMemory = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).TotalMemory
	echo $BladeTotalMemory | export-excel $filename -worksheetname $worksheetname -startcolumn 8 -startrow 9 -AutoSize
	
	$BladeNumOfCpus = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).NumOfCpus
	echo $BladeNumOfCpus | export-excel $filename -worksheetname $worksheetname -startcolumn 9 -startrow 9 -AutoSize
	
	$BladeNumOfCores = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).NumOfCores
	echo $BladeNumOfCores | export-excel $filename -worksheetname $worksheetname -startcolumn 10 -startrow 9 -AutoSize
	 
	$BladeMkt = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) } ).Ucs
	echo $BladeMkt | export-excel $filename -worksheetname $worksheetname -startcolumn 11 -startrow 9 -AutoSize

}
Disconnect-Ucs
}

