#Import required modules
Import-Module ImportExcel

#Capture process start time
$startTime = get-date 

#Define global variables
$date = get-date -format yyyy-MM-dd
$time = get-date -uformat %R
$filename = ".\CPU_Ready_" +"$date.csv"
$excelfilename = ".\CPU_Ready_" +"$date.xlsx"
#$worksheetname = $date

#Define vCenters
$dtwdr = 'dtw-dr-vc01.dtw.vdc.local'
$dtwmgmt = 'dtw-mgmt-vc.dtw.vdc.local'
$kwddr = 'kwd-dr-vc01.dtw.vdc.local'
$kwdmgmt = 'kwd-mgmt-vc.dtw.vdc.local'
$vc01 = 'vmw-mgmt-vc01.grr.vdc.local'
$vc02 = 'vmw-mgmt-vc02.vdc.local'
$vc05 = 'vmw-mgmt-vc05.grr.vdc.local'
$vc06 = 'vmw-mgmt-vc06.dtw.vdc.local'
$vCs = $dtwdr,$dtwmgmt,$kwddr,$kwdmgmt,$vc01,$vc02,$vc05,$vc06

#Select which vCenter to connect to
$countVC = 0
foreach($vC in $vCs){
Write-Output "[$countVC] $Vc"
$countVC = $countVC+1
}
write-output " "
$vcchoice = Read-Host "Which vCenter would you like to connect to?"
Write-Output " "
#Connect to the Desired vCenter
$connect = connect-viserver $vcs[$vcchoice]
Write-output "Connection established"
write-output " "

#Select the cluster for data gathering
$Cluster = Get-Cluster
$countCL = 0
Write-Output " "
Write-Output "Clusters: "
Write-Output " "
foreach($oC in $Cluster){
Write-Output "[$countCL] $oc"
$countCL = $countCL+1
}
Write-Output " "
$choice = Read-Host "Which cluster would you like to review?"
Write-Output " "
Write-host "please wait for script to finish, it may take some minutes." -ForegroundColor Yellow
$cluster = get-cluster $cluster[$choice]
$vms = (Get-cluster $cluster | Get-VM | where {$_.PowerState -eq "PoweredOn"}).Name
$worksheetname = $cluster

#Collect and organize data
$returnObj = @()
ForEach ($VMName in $VMs)
{       
	   $cpus = (Get-VM $VMName).NumCPU
	   $mem = (Get-VM $VMName).MemoryGB
	   $vmlimit = (Get-VM $VMName | Get-VMResourceConfiguration).CPULimitMHz
	   $vmhost = (Get-VM $VMName).VmHost
	   $HostCPU = (Get-VMHost $vmhost).NumCPU
	   $hostusedmhz = (Get-VMHOst $vmhost).CpuUsageMhz
	   $hosttotalmhz = (Get-VMHOst $vmhost).CpuTotalMhz
	   $stats = get-vm $VMName | where {$_.PowerState -eq "PoweredOn"} | Get-Stat -realtime -maxsamples 1 -Stat Cpu.Ready.Summation -Instance ""
	   $aggr_stats = $stats | Measure-Object -Property Value -Average 
       $ready = [Math]::Round(((($aggr_stats.Average)/(20*1000) * 100)),2)
	   $rdy = [Math]::Round(($ready/$cpus),2)
	
	   $obj = new-object psobject -Property @{
				Date = $date
				Time = $time
				VMName = $VMName
				vCPUs = $cpus
				CPUReady = $rdy
				VMLimit = $vmlimit
				VMHost = $vmhost
				HostCPUCount = $hostcpu
				HostCPUUsedMHz = $hostusedmhz
				HostCPUTotalMHz = $hosttotalmhz 
				}
		$returnObj += $obj | Select Date,Time,VMName,vCPUs,VMLimit,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz | where ({$_.CPUReady -ne "0"})
}
#Paramters for sorting returned values
$prop1 = @{Expression='CPUReady'; Descending=$true }
$prop2 = @{Expression='VMLimit'; Ascending=$true }

#Return collected data
write-output $returnObj | Select Date,Time,VMName,vCPUs,VMLimit,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz |  Sort-Object $prop1, $prop2 #| export-excel $excelfilename -worksheetname $worksheetname -startcolumn 2 -startrow 3 -append -Autosize 
# write-output $returnObj | Select Date,Time,VMName,vCPUs,VMLimit,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz |  Sort-Object -Property CPUReady -Descending #| export-excel $excelfilename -worksheetname $worksheetname -startcolumn 2 -startrow 3 -append -Autosize 

#Calculate time required to execute script
$EndTime = get-date 
$Duration = $EndTime-$StartTime
echo "$duration elapsed"
echo " "
echo "Press enter to exit..."
pause

#Disconnect from all vCenters
Disconnect-VIServer -Server * -Force -Confirm:$false
