#Import required modules
#Import-Module ImportExcel

#Capture process start time
$startTime = get-date 

#Define global variables
$date = get-date -format yyyy-MM-dd
$time = get-date -uformat %R

#Define vCenters
$dtwdr = 'dtw-dr-vc01.dtw.vdc.local'
$dtwmgmt = 'dtw-mgmt-vc.dtw.vdc.local'
$kwddr = 'kwd-dr-vc01.kwd.vdc.local'
$kwdmgmt = 'kwd-mgmt-vc.kwd.vdc.local'
$vc01 = 'vmw-mgmt-vc01.grr.vdc.local'
$vc02 = 'vmw-mgmt-vc02.vdc.local'
$vc05 = 'vmw-mgmt-vc05.grr.vdc.local'
$vc06 = 'vmw-mgmt-vc06.dtw.vdc.local'
$vCenters = $dtwdr,$dtwmgmt,$kwddr,$kwdmgmt,$vc01,$vc02,$vc05,$vc06

cls

#Select which vCenter to connect to
$countVC = 0
Write-Output " "
foreach($vC in $vCenters){
Write-Output "[$countVC] $vC"
$countVC = $countVC+1
}
write-output " "
$choice = Read-Host "Which vCenter would you like to connect to?"
Write-Output " "
#Connect to the Desired vCenter
$connection = connect-viserver $vCenters[$choice]
Write-output "Connection established"
write-output " "
$choice = 0
cls

#Select the cluster for data gathering
$Cluster = (Get-Cluster | Sort-Object -Property Name).Name
$countCL = 0
Write-Output " "
Write-Output "Clusters: "
Write-Output " "
foreach($oC in $Cluster){
Write-Output "[$countCL] $oc"
$countCL = $countCL+1
}
Write-Output " "
$choice = Read-Host "Which cluster would you like to review?"
Write-Output " "
# Write-host "please wait for script to finish, it may take some minutes." -ForegroundColor Yellow
$cluster = get-cluster $cluster[$choice]
$choice = 0

cls
Write-Output " "
Write-host "Please wait for script to finish, it may take some minutes." -ForegroundColor Yellow
Write-Output " "

#GetClusterStats
$stat = 'cpu.ready.summation'
$esxhosts = Get-Cluster -Name $cluster | Get-VMHost
$stats = Get-Stat -Entity $esxhosts -Stat $stat -Realtime -MaxSamples 1 -Instance ""

#Calculate Individual Host Stats
$returnObj = @()
ForEach ($esxhost in $esxhosts){
$esxreadyAvg = Get-Stat -Entity $esxhost -Stat $stat -Realtime -MaxSamples 1 -Instance "" | Measure-Object -Property Value -Average | select -ExpandProperty Average
$esxreadyPerc = $esxreadyAvg / ($stats[0].IntervalSecs * 1000)
$esxrdy = [Math]::Round(($esxreadyPerc),2)
$esxrdyformat = "$esxrdy" + "%"
				$obj = new-object psobject -Property @{
				VMHost = $esxhost
				CPUReady = $esxrdyformat
				}
				$returnObj += $obj | Select VMhost,CPUReady
}
write-output $returnObj | Sort-Object -Property VMhost

#Calculate Cluster Stats
$clusterreadyAvg = $stats | Measure-Object -Property Value -Average | select -ExpandProperty Average
$clusterreadyPerc = $clusterreadyAvg / ($stats[0].IntervalSecs * 1000)
#Write-Output "Cluster $($clusterName) - CPU ready % $('{0:p}' -f $readyPerc)"
write-output " "
$clusterrdy = [Math]::Round(($clusterreadyPerc),2)
echo "Cluster Ready Percentage:"
Write-Output $clusterrdy"%"

#Calculate time required to execute script
$EndTime = get-date 
$Duration = $EndTime-$StartTime
echo " "
echo "Operation took $duration to complete"
#Disconnect from all vCenters
Disconnect-VIServer -Server * -Force -Confirm:$false

