$RPs = Get-ResourcePool | Where {($_.Name -ne 'Resources')}

$RPObject= @()
Foreach($RP in $RPs){
$rpifno = Get-Resourcepool $rp |  Select Name,Parent,CpuLimitMhz,CPUReservationMhz,CPuExpandableReservation,MemLimitGB,MemExpandableReservation,MemReservationGB,@{N='OverallCpuUsage';E={$_.ExtensionData.Summary.QuickStats.OverallCpuUsage}},@{N='OverallCpuDemand';E={$_.ExtensionData.Summary.QuickStats.OverallCpuDemand}},@{N='GuestMemoryUsage';E={$_.ExtensionData.Summary.QuickStats.GuestMemoryUsage}},@{N='HostMemoryUsage';E={$_.ExtensionData.Summary.QuickStats.HostMemoryUsage}},@{N='DistributedCpuEntitlement';E={$_.ExtensionData.Summary.QuickStats.DistributedCpuEntitlement}},@{N='DistributedMemoryEntitlement';E={$_.ExtensionData.Summary.QuickStats.DistributedMemoryEntitlement}},@{N='StaticCpuEntitlement';E={$_.ExtensionData.Summary.QuickStats.StaticCpuEntitlement}},@{N='StaticMemoryEntitlement';E={$_.ExtensionData.Summary.QuickStats.StaticMemoryEntitlement}},@{N='SwappedMemory';E={$_.ExtensionData.Summary.QuickStats.SwappedMemory}},@{N='BalloonedMemory';E={$_.ExtensionData.Summary.QuickStats.BalloonedMemory}}


$rpname = $RP
$parent = ($rpname).Parent
$CPULimit = ($rpifno).CPULimitMHz
$CPUReservation = ($rpname).CPUReservationMHz
$CPUExpandable = ($rpname).CPUExpandableReservation
$vCPUs = (Get-VM -location $RP | Where {($_.Powerstate -eq "PoweredOn")}).NumCPU | Measure-Object -Sum | select -ExpandProperty Sum
$vCPUReservation = ((Get-VM -location $RP).ExtensionData.ResourceConfig.CpuAllocation).Reservation | Measure-Object -Sum | select -ExpandProperty Sum
$MemReservation = ($rpname).MemReservationGB
$MemExpandable = ($rpname).MemExpandableReservation
$MemLimitGB = ($rpname).MemLimitGB				
$OverallCpuUsage = ($rpifno).OverallCpuUsage
$OverallCpuDemand = ($rpifno).OverallCpuDemand
$GuestMemoryUsage = ($rpifno).GuestMemoryUsage
$HostMemoryUsage = ($rpifno).HostMemoryUsage
$DistributedCpuEntitlement = ($rpifno).DistributedCpuEntitlement
$DistributedMemoryEntitlement = ($rpifno).DistributedMemoryEntitlement
$StaticCpuEntitlement = ($rpifno).StaticCpuEntitlement
$StaticMemoryEntitlement = ($rpifno).StaticMemoryEntitlement
$SwappedMemory = ($rpifno).SwappedMemory
$BalloonedMemory = ($rpifno).BalloonedMemory

$RPObj = new-object psobject -Property @{
		ResourcePool = $rpname
		OverallCpuUsage = $OverallCpuUsage
		OverallCpuDemand = $OverallCpuDemand
		CPUReservation = $CPUReservation
		CPULimit = $CPULimit
		vCPUs = $vCPUs
		CPUReserved = $vCPUReservation
		CPUExpandable = $CPUExpandable
		MemReservation = $MemReservation
		MemLimit = $MemLimit
		MemExpandable = $MemExpandable
		GuestMemoryUsage = $GuestMemoryUsage
		HostMemoryUsage = $HostMemoryUsage
		DistributedCpuEntitlement = $DistributedCpuEntitlement
		DistributedMemoryEntitlement = $DistributedMemoryEntitlement
		StaticCpuEntitlement = $StaticCpuEntitlement
		StaticMemoryEntitlement = $StaticMemoryEntitlement
		SwappedMemory = $SwappedMemory
		BalloonedMemory = $BalloonedMemory
				}
$RPObject += $rpObj
}
$rpfinal = $rpObject
#Output CPU Info
write-output $RPfinal | Select ResourcePool,CPUReservation,CPULimit,CPUReserved,vCPUs,CPUExpandable,OverallCpuUsage,OverallCpuDemand,DistributedCpuEntitlement,StaticCpuEntitlement | Export-CSV C:\users\tverkade\desktop\RP_CPU_Info.csv -notypeinformation

#Output Memory Info
write-output $RPFinal | Select ResourcePool,MemReservation,MemLimit,MemExpandable,GuestMemoryUsage,HostMemoryUsage,DistributedMemoryEntitlement,StaticMemoryEntitlement,SwappedMemory,BalloonedMemory | Export-CSV C:\users\tverkade\desktop\RP_Mem_Info.csv -notypeinformation