#Get Clusters
$clusters = Get-Cluster | where {($_.Name -ne 'Staging') -and ($_.Name -notmatch 'Onboarding')} 

$clobject= @()
Foreach($cluster in $clusters){
$clustersummary = Get-Cluster $cluster | Get-View | Select Name, @{Name="TotalCpuCapacityMhz";Expression={$_.Summary.UsageSummary.TotalCpuCapacityMhz}},@{Name="TotalMemCapacityMB";Expression={$_.Summary.UsageSummary.TotalMemCapacityMB}},@{Name="CpuReservationMhz";Expression={$_.Summary.UsageSummary.CpuReservationMhz}},@{Name="MemReservationMB";Expression={$_.Summary.UsageSummary.MemReservationMB}},@{Name="PoweredOffCpuReservationMhz";Expression={$_.Summary.UsageSummary.PoweredOffCpuReservationMhz}},@{Name="PoweredOffMemReservationMB";Expression={$_.Summary.UsageSummary.PoweredOffMemReservationMB}},@{Name="CpuDemandMhz";Expression={$_.Summary.UsageSummary.CpuDemandMhz}},@{Name="MemDemandMB";Expression={$_.Summary.UsageSummary.MemDemandMB}},@{Name="StatsGenNumber";Expression={$_.Summary.UsageSummary.StatsGenNumber}},@{Name="CpuEntitledMhz";Expression={$_.Summary.UsageSummary.CpuEntitledMhz}},@{Name="MemEntitledMB";Expression={$_.Summary.UsageSummary.MemEntitledMB}},@{Name="PoweredOffVmCount";Expression={$_.Summary.UsageSummary.PoweredOffVmCount}},@{Name="TotalVmCount";Expression={$_.Summary.UsageSummary.TotalVmCount}},@{Name="TargetBalance";Expression={$_.Summary.TargetBalance}},@{Name="CurrentBalance";Expression={$_.Summary.CurrentBalance}},@{Name="NumberOfHosts";Expression={$_.Summary.NumHosts}}

#Define individual stats
$ClusterName = ($Cluster).Name
$TotalCpuCapacityMhz = $clustersummary.TotalCpuCapacityMhz
$TotalMemCapacityMB = $clustersummary.TotalMemCapacityMB
$CpuReservationMhz = $clustersummary.CpuReservationMhz
$MemReservationMB = $clustersummary.MemReservationMB
$CpuDemandMhz = $clustersummary.CpuDemandMhz
$MemDemandMB = $clustersummary.MemDemandMB
$CpuEntitledMhz = $clustersummary.CpuEntitledMhz
$MemEntitledMB = $clustersummary.MemEntitledMB
$CpuFreeAfterRes = [math]::round((($TotalCpuCapacityMhz-($CpuReservationMhz))/$TotalCpuCapacityMhz)*100,2)
$MemFreeAfterRes = [math]::round((($TotalMemCapacityMB-$MemReservationMB)/$TotalMemCapacityMB)*100,2)
$CpuFree = [math]::round((($TotalCpuCapacityMhz-($CpuReservationMhz))/$TotalCpuCapacityMhz),2)
$CpuFreeEnt = [math]::round((($TotalCpuCapacityMhz-($CpuEntitledMhz))/$TotalCpuCapacityMhz),2)
$NumHosts = $clustersummary.NumberofHosts
$HostCPUCap = [math]::round(($TotalCpuCapacityMhz/$numhosts),0)
$HostMemCap = [math]::round(($TotalMemCapacityMB/$numhosts),0)
$HostFailureDomCpuRes = [math]::round((($TotalCpuCapacityMhz * $CPUFree)/($HostCPUCap)),0)
$HostFailureDomCpuEnt = [math]::round((($TotalCpuCapacityMhz * $CPUFreeEnt)/($HostCPUCap)),0)
$HostFailureDomCpu = ($HostFailureDomCpuRes, $HostFailureDomCpuEnt |Measure-Object -Minimum).Minimum
$HostFailureDomainCpu = ($HostFailureDomCpu-1)
$HostFailureDomMem = [math]::round((($TotalMemCapacityMB-$MemReservationMb)/($HostMemCap)),0)
$HostFailureDomainMem = ($HostFailureDomMem-1)
$HostFailureDomain = ($HostFailureDomainMem, $HostFailureDomainCPU |Measure-Object -Minimum).Minimum
$CpuFreeAfterReservation = "$CpuFreeAfterRes" + "%"
$MemFreeAfterReservation = "$MemFreeAfterRes" + "%"
$PoweredOffCpuReservationMhz = $clustersummary.PoweredOffCpuReservationMhz
$PoweredOffMemReservationMB = $clustersummary.PoweredOffMemReservationMB
$TargetBalance = $clustersummary.TargetBalance
$CurrentBalance = $clustersummary.CurrentBalance
$clusterVMs = (Get-Cluster $cluster | Get-Vm)
$PoweredOffVmCount = ($clusterVMs | where {($_.PowerState -eq 'PoweredOff')}).Name | Measure-Object | select -ExpandProperty Count
$TotalVmCount = ($clusterVMs ).Name | Measure-Object | select -ExpandProperty Count
$TotalVMCPUs = ($clusterVMs ).NumCPU | Measure-Object -Sum | select -ExpandProperty Sum
$TotalHostCPUs = (Get-Cluster $cluster | Get-VmHost).NumCPU | Measure-Object -Sum | select -ExpandProperty Sum
$CPUOverSub = [math]::round(($TotalVMCPUs/$TotalHostCPUs),2)
$CPUOverSubscription = "$CPUOverSub" + " to 1"

#Combine collected data for each cluster into new object
$clobj = new-object psobject -Property @{
			ClusterName = $ClusterName
			TotalCpuCapacityMhz = $TotalCpuCapacityMhz
			TotalMemCapacityMB = $TotalMemCapacityMB
			CpuReservationMhz = $CpuReservationMhz
			CPUFreeAfterReservation = $CPUFreeAfterReservation
			MemFreeAfterReservation = $MemFreeAfterReservation
			MemReservationMB = $MemReservationMB
			PoweredOffCpuReservationMhz = $PoweredOffCpuReservationMhz
			PoweredOffMemReservationMB = $PoweredOffMemReservationMB
			CpuDemandMhz = $CpuDemandMhz
			HostFailureDomainCPU = $HostFailureDomainCpu
			HostFailureDomainMem = $HostFailureDomainMem
			HostFailureDomain = $HostFailureDomain
			MemDemandMB = $MemDemandMB
			CpuEntitledMhz = $CpuEntitledMhz
			MemEntitledMB = $MemEntitledMB
			TargetBalance = $TargetBalance
			CurrentBalance = $CurrentBalance
			PoweredOffVmCount = $PoweredOffVmCount
			TotalVmCount = $TotalVmCount
			TotalVMCPUs = $TotalVMCPUs
			TotalHostCPUs = $TotalHostCPUs
			CPUOverSubscription = $CPUOverSubscription
			HostCPUCap = $HostCPUCap
			HostMemCap = $HostMemCap
			NumHosts = $NumHosts
				}
$ClObject += $clObj
}
$clusterfinal = $ClObject
#Output Cluster Info
echo "Overall Cluster Information"
#write-output $clusterfinal | Select ClusterName,TotalCpuCapacityMhz,CpuReservationMhz,CpuDemandMhz,CpuEntitledMhz,PoweredOffVmCount,TotalVmCount,TotalVMCPUs,TotalHostCPUs,CPUOverSubscription | Export-CSV .\Cluster_CPU_Info.csv -notypeinformation
write-output $clusterfinal | Select ClusterName,HostFailureDomain,TargetBalance,CurrentBalance,NumHosts,TotalVmCount,PoweredOffVmCount | FT -autosize

#Output CPU Info
echo "Cluster CPU Information"
#write-output $clusterfinal | Select ClusterName,TotalCpuCapacityMhz,CpuReservationMhz,CpuDemandMhz,CpuEntitledMhz,PoweredOffVmCount,TotalVmCount,TotalVMCPUs,TotalHostCPUs,CPUOverSubscription | Export-CSV .\Cluster_CPU_Info.csv -notypeinformation
write-output $clusterfinal | Select ClusterName,TotalCpuCapacityMhz,CpuReservationMhz,CPUFreeAfterReservation,HostCPUCap,CpuDemandMhz,CpuEntitledMhz,TotalVMCPUs,TotalHostCPUs,CPUOverSubscription | FT -autosize

#Output Memory Info
echo "Cluster Memory Information"
#write-output $clusterfinal | Select ClusterName,TotalMemCapacityMB,MemReservationMB,MemDemandMB,MemEntitledMB,PoweredOffVmCount,TotalVmCount | Export-CSV .\Cluster_Mem_Info.csv -notypeinformation
write-output $clusterfinal | Select ClusterName,TotalMemCapacityMB,MemReservationMB,MemFreeAfterReservation,HostMemCap,MemDemandMB,MemEntitledMB | FT -autosize