#Import required modules
#Import-Module ImportExcel

#Capture process start time
#$startTime = get-date 

#Define global variables
$date = get-date -format yyyy-MM-dd
$time = get-date -uformat %R

#Define vCenters
$dtwdr = 'dtw-dr-vc01.dtw.vdc.local'
$dtwmgmt = 'dtw-mgmt-vc.dtw.vdc.local'
$kwddr = 'kwd-dr-vc01.kwd.vdc.local'
$kwdmgmt = 'kwd-mgmt-vc.kwd.vdc.local'
$vc01 = 'vmw-mgmt-vc01.grr.vdc.local'
$vc02 = 'vmw-mgmt-vc02.vdc.local'
$vc05 = 'vmw-mgmt-vc05.grr.vdc.local'
$vc06 = 'vmw-mgmt-vc06.dtw.vdc.local'
$vCenters = $dtwdr,$dtwmgmt,$kwddr,$kwdmgmt,$vc01,$vc02,$vc05,$vc06

cls

#Select which vCenter to connect to
$countVC = 0
Write-Output " "
foreach($vC in $vCenters){
Write-Output "[$countVC] $vC"
$countVC = $countVC+1
}
write-output " "
$choice = Read-Host "Which vCenter would you like to connect to?"
Write-Output " "
#Connect to the Desired vCenter
$connection = connect-viserver $vCenters[$choice]
$vc = $vCenters[$choice]
Write-output "Connection established"
write-output " "
$choice = 0

$filename = ".\" + "$vc" + ".csv"

$cluster = (get-cluster).Name
$choice = 0
$esxhosts = Get-VMhost

#Capture process start time
$startTime = get-date 

#Collect ESX host driver information
$returnObj = @()
ForEach ($esxhost in $esxhosts){
$esxcli = Get-EsxCli -VMhost $esxhost
$nic0 = $esxcli.network.nic.get("vmnic0").DriverInfo | select Driver, FirmwareVersion, Version
$nic1 = $esxcli.network.nic.get("vmnic1").DriverInfo | select Driver, FirmwareVersion, Version
	$obj = new-object psobject -Property @{
		 Cluster = $cluster
		 Host = $esxhost
		 Nic0Name = 'Nic 0'
		 Nic0Driver = $nic0.Driver
		 Nic0FwVer = $nic0.FirmwareVersion
		 Nic0Ver = $nic0.Version
		 Nic1Name = 'Nic 1'
		 Nic1Driver = $nic1.Driver
		 Nic1FwVer = $nic1.FirmwareVersion
		 Nic1Ver = $nic1.Version
		}
		$returnObj += $obj | Select Host,Cluster,Nic0Name,Nic0Driver,Nic0FwVer,Nic0Ver,Nic1Name,Nic1Driver,Nic1FwVer,Nic1Ver
}
write-output $returnObj | Sort-Object -Property Host | ft -autosize
write-output $returnObj | Sort-Object -Property Host | Export-CSV $filename -append -notypeinformation

#Calculate time required to execute script
$EndTime = get-date 
$Duration = $EndTime-$StartTime
echo " "
echo "Operation took $duration to complete"

#Disconnect from all vCenters
Disconnect-VIServer -Server * -Force -Confirm:$false


