#Select the cluster for data gathering
$Cluster = (Get-Cluster | Sort-Object -Property Name).Name
$countCL = 0
Write-Output " "
Write-Output "Clusters: "
Write-Output " "
foreach($oC in $Cluster){
Write-Output "[$countCL] $oc"
$countCL = $countCL+1
}
Write-Output " "
$choice = Read-Host "Which cluster would you like to review?"
Write-Output " "
$cluster = get-cluster $cluster[$choice]
$choice = 0

#Select the top-level resource pool for data gathering
$PRPs = (Get-resourcePool -norecursion -location (Get-Cluster $cluster | Get-ResourcePool 'Resources') | Sort-Object -Property Name).Name
$countPRP = 0
Write-Output " "
Write-Output "Parent resource pools: "
Write-Output " "
foreach($oP in $PRPs){
Write-Output "[$countPRP] $oP"
$countPRP = $countPRP+1
}
Write-Output " "
$PRPchoice = Read-Host "Please select the parent resource pool"
Write-Output " "
# $selectedprp = Get-resourcepool $PRPs[$PRPchoice]
$selectedprp = (Get-resourcePool $PRPs[$PRPchoice]).Name

#Get Host CPU total MHz for cluster
$ESXHosts = Get-Cluster $cluster | Get-VMHost
$ESXComputeTotal = @()
ForEach ($EsxHost in $ESXhosts){
$ESXCompute = (Get-VMHost $ESXHost).CpuTotalMHz
$ESXComputeTotal += $ESXCompute
}
$ClusterCompute = ($ESXComputeTotal | Measure-Object -Sum).Sum

#Get Host CPU MHz used numbers for cluster
$ESXHosts = Get-Cluster $cluster | Get-VMHost
$ESXComputeUsedTotal = @()
ForEach ($EsxHost in $ESXhosts){
$ESXComputeUsed = (Get-VMHost $ESXHost).cpuusagemhz
$ESXComputeUsedTotal += $ESXComputeUsed
}
$ClusterComputeUsed = ($ESXComputeUsedTotal | Measure-Object -Sum).Sum


#Get Non-empty Flexible Resource Pools
$FRPs = (Get-ResourcePool -location (Get-ResourcePool $selectedprp) | Where-Object {($_.CpuExpandableReservation -match "FALSE") -and ($_.MemExpandableReservation -match "TRUE")})


$frpobject = @()
ForEach ($FRP in $FRPs){
$FRPName = (Get-Resourcepool $FRP).Name
$FRPComputeUsed = (Get-ResourcePool $FRP | Get-View | Select @{Name="CPUUsage";Expression={$_.summary.quickstats.OverallCPuUsage}}).CPUUsage
$FRPComputeReserved = (Get-ResourcePool $FRP).CpuReservationMHz
$FRPComputeLimit = (Get-ResourcePool $FRP | Where-Object {($_.CpuLimitMHz -ne "-1")}).CpuLimitMHz
$FRPComputeUsedTotal += $FRPComputeUsed
$FRPComputeReservedTotal += $FRPComputeReserved
$FRPComputeLimitTotal += $FRPComputeLimit
$FRPObj = new-object psobject -Property @{
				FRPName = $FRPName
				FRPComputeUsed = $FRPComputeUsed
				FRPComputeReserved = $FRPComputeReserved	
				FRPComputeLimit = $FRPComputeLimit
				}
		$frpobject += $frpobj #| where 
}
$TotalFRPComputeUsed = ($FRPComputeUsedTotal | Measure-Object -Sum).Sum
$TotalFRPComputeReserved = ($FRPComputeReservedTotal | Measure-Object -Sum).Sum
$TotalFRPComputeLimit = ($FRPComputeLimitTotal | Measure-Object -Sum).Sum

$FRPReturned = $FRPObject
$FRPReturnedMetrics = "FRPName","FRPComputeUsed","FRPComputeReserved","FRPComputeLimit"

#Export FRP info to CSV
$frpcsvfilename = "$cluster" + "_FRP_info.csv"
write-output $FRPreturned | Select $FRPReturnedMetrics | export-csv $frpcsvfilename -notypeinformation


#Get Non-empty Flexible Resource Pools
$DRPs =  (Get-ResourcePool -location (Get-ResourcePool $selectedprp) | Where-Object {($_.CpuExpandableReservation -match "FALSE") -and ($_.MemExpandableReservation -match "FALSE")})

$DRPObject = @()
ForEach ($DRP in $DRPs){
$DRPName = (Get-Resourcepool $DRP).Name
$DRPComputeUsed = (Get-ResourcePool $DRP | Get-View | Select @{Name="CPUUsage";Expression={$_.summary.quickstats.OverallCPuUsage}}).CPUUsage
$DRPComputeReserved = (Get-ResourcePool $DRP).CpuReservationMHz
$DRPComputeUsedTotal += $DRPComputeUsed
$DRPComputeReservedTotal += $DRPComputeReserved
$DRPObj = new-object psobject -Property @{
				DRPName = $DRPName
				DRPComputeUsed = $DRPComputeUsed
				DRPComputeReserved = $DRPComputeReserved	
				DRPComputeLimit = $DRPComputeLimit
				}
		$Drpobject += $drpobj
}
$TotalDRPComputeUsed = ($DRPComputeUsedTotal | Measure-Object -Sum).Sum
$TotalDRPComputeReserved = ($DRPComputeReservedTotal | Measure-Object -Sum).Sum
$DRPReturned = $DRPObject


#Export DRP info to CSV
$drpcsvfilename = "$cluster" + "_DRP_info.csv"
write-output $DRPreturned | export-csv $drpcsvfilename -notypeinformation




#Get PAYG Pool Info
$PAYGs = (Get-ResourcePool -location (Get-ResourcePool $selectedprp) | Where-Object {($_.CpuExpandableReservation -match "TRUE") -and ($_.MemExpandableReservation -match "TRUE")})

$PAYGObject = @()
ForEach ($PAYG in $PAYGs){
$FRPName = (Get-Resourcepool $DRP).Name
$PAYGComputeUsed = (Get-ResourcePool $PAYG | Get-View | Select @{Name="CPUUsage";Expression={$_.summary.quickstats.OverallCPuUsage}}).CPUUsage
$FRPComputeReserved = (Get-ResourcePool $PAYG).CpuReservationMHz
$PAYGComputeUsedTotal += $PAYGComputeUsed
$PAYGComputeReservedTotal += $PAYGComputeReserved
$PAYGObj = new-object psobject -Property @{
				PAYGName = $PAYGName
				PAYGComputeUsed = $PAYGComputeUsed
				PAYGComputeReserved = $PAYGComputeReserved	
				PAYGComputeLimit = $PAYGComputeLimit
				}
		$PAYGobject += $paygobj
}
$TotalPAYGComputeUsed = ($PAYGComputeUsedTotal | Measure-Object -Sum).Sum
$TotalPAYGComputeReserved = ($PAYGComputeReservedTotal | Measure-Object -Sum).Sum
$PAYGReturned = $PAYGObject


#Export PAYG info to CSV
$paygcsvfilename = "$cluster" + "_PAYG_info.csv"
write-output $PAYGreturned | export-csv $paygcsvfilename -notypeinformation


#Calculate Percentages
$ClusterComputePercentUsed = [Math]::Round((($ClusterComputeUsed/$ClusterCompute)*100),2)
$TotalFRPComputePercentUsed = [Math]::Round((($TotalFRPComputeUsed/$ClusterCompute)*100),2)
$TotalDRPComputePercentUsed = [Math]::Round((($TotalDRPComputeUsed/$ClusterCompute)*100),2)
$TotalPAYGComputePercentUsed = [Math]::Round((($TotalPAYGComputeUsed/$ClusterCompute)*100),2)

#Calculate Total Serverations
$TotalComputeReserved = [Math]::Round(($TotalFRPComputeReserved + $TotalDRPComputeReserved + $TotalPAYGComputeReserved),2)
$TotalComputeUnreserved = [Math]::Round(($ClusterCompute - $TotalComputeReserved),2)


#Combine Info
  $ComputeObj = new-object psobject -Property @{
				ClusterComputeUsed = $ClusterComputeUsed
				ClusterComputePercentUsed = $ClusterComputePercentUsed
				ClusterComputeTotal = $ClusterCompute
				TotalComputeReserved = $TotalComputeReserved
				RemainingComputeUnreserved = $TotalComputeUnreserved
				TotalFRPComputeUsed = $TotalFRPComputeUsed
				TotalFRPComputeReserved = $TotalFRPComputeReserved
				TotalFRPComputePercentUsed = $TotalFRPComputePercentUsed
				TotalFRPComputeLimit = $TotalFRPComputeLimit
				TotalDRPComputeUsed = $TotalDRPComputeUsed
				TotalDRPComputeReserved = $TotalDRPComputeReserved
				TotalDRPComputePercentUsed = $TotalDRPComputePercentUsed
				TotalPAYGComputeUsed = $TotalPAYGComputeUsed
				TotalPAYGComputePercentUsed = $TotalPAYGComputePercentUsed
				TotalPAYGComputeReserved = $TotalPaygComputeReserved

								
				}
				
$clusterReturnedMetrics = "ClusterComputeUsed","ClusterComputePercentUsed","ClusterComputeTotal","TotalComputeReserved","RemainingComputeUnreserved"
$FRPReturnedMetrics = "TotalFRPComputeUsed","TotalFRPComputeReserved","TotalFRPComputeLimit"#,"TotalFRPComputePercentUsed"
$DRPReturnedMetrics = "TotalDRPComputeUsed","TotalDRPComputeReserved"#,"TotalDRPComputePercentUsed"
$PAYGReturnedMetrics = "TotalPAYGComputeUsed","TotalPAYGComputeReserved"#,"TotalPAYGComputePercentUsed"

$computeReturned = $ComputeObj 

				write-output $computeReturned | Select $clusterReturnedMetrics | ft -autosize
				write-output $computeReturned | Select $FRPReturnedMetrics | ft -autosize
				write-output $computeReturned | Select $DRPReturnedMetrics | ft -autosize
				write-output $computeReturned | Select $PAYGReturnedMetrics | ft -autosize

