#Select the cluster for data gathering
$Cluster = (Get-Cluster | Sort-Object -Property Name).Name
$countCL = 0
Write-Output " "
Write-Output "Clusters: "
Write-Output " "
foreach($oC in $Cluster){
Write-Output "[$countCL] $oc"
$countCL = $countCL+1
}
Write-Output " "
$choice = Read-Host "Which cluster would you like to review?"
Write-Output " "
$cluster = get-cluster $cluster[$choice]
$choice = 0

#Select the top-level resource pool for data gathering
$PRPs = (Get-resourcePool -norecursion -location (Get-Cluster $cluster | Get-ResourcePool 'Resources') | Sort-Object -Property Name).Name
$countPRP = 0
Write-Output " "
Write-Output "Parent resource pools: "
Write-Output " "
foreach($oP in $PRPs){
Write-Output "[$countPRP] $oP"
$countPRP = $countPRP+1
}
Write-Output " "
$PRPchoice = Read-Host "Please select the parent resource pool"
Write-Output " "
# $selectedprp = Get-resourcepool $PRPs[$PRPchoice]
$selectedprp = (Get-resourcePool $PRPs[$PRPchoice]).Name

#Get Host CPU total MHz for cluster
$ESXHosts = Get-Cluster $cluster | Get-VMHost
$ESXComputeTotal = @()
ForEach ($EsxHost in $ESXhosts){
$ESXCompute = (Get-VMHost $ESXHost).CpuTotalMHz
$ESXComputeTotal += $ESXCompute
}
$ClusterCompute = ($ESXComputeTotal | Measure-Object -Sum).Sum

#Get Host CPU MHz used numbers for cluster
$ESXHosts = Get-Cluster $cluster | Get-VMHost
$ESXComputeUsedTotal = @()
ForEach ($EsxHost in $ESXhosts){
$ESXComputeUsed = (Get-VMHost $ESXHost).cpuusagemhz
$ESXComputeUsedTotal += $ESXComputeUsed
}
$ClusterComputeUsed = ($ESXComputeUsedTotal | Measure-Object -Sum).Sum


#Get Non-empty Flexible Resource Pools
$FRPs = (Get-ResourcePool -location (Get-ResourcePool $selectedprp) | Where-Object {($_.CPULimitMHz -ne "0") -and ($_.MemLimitGB -eq "-1") -and ($_.CPUReservationMHz -ne "0") -and ($_.Name -notmatch "Resource")}).Name 

$TotalFRPComputeUsed = @()
ForEach ($FRP in $FRPs){
$FRPComputeUsed = (Get-ResourcePool $FRP | Get-View | Select @{Name="CPUUsage";Expression={$_.summary.quickstats.OverallCPuUsage}}).CPUUsage
$FRPComputeReserved = (Get-ResourcePool $FRP | Get-View | Select @{Name="MHzReserved";Expression={$_.summary.quickstats.distributedCpuEntitlement}}).MHzReserved
$FRPComputeUsedTotal += $FRPComputeUsed
$FRPComputeReservedTotal += $FRPComputeReserved
}
$TotalFRPComputeUsed = ($FRPComputeUsedTotal | Measure-Object -Sum).Sum
$TotalFRPComputeReserved = ($FRPComputeReservedTotal | Measure-Object -Sum).Sum

#Get Non-empty Flexible Resource Pools
$DRPs =  Get-ResourcePool -location $selectedprp | Where-Object {($_.CPULimitMHz -ne "-1") -and ($_.MemLimitGB -ne "-1") -and ($_.Name -notmatch "Resource")}

$TotalDRPComputeUsed = @()
ForEach ($DRP in $DRPs){
$DRPComputeUsed = (Get-ResourcePool $DRP | Get-View | Select @{Name="CPUUsage";Expression={$_.summary.quickstats.OverallCPuUsage}}).CPUUsage
$DRPComputeReserved = (Get-ResourcePool $DRP | Get-View | Select @{Name="MHzReserved";Expression={$_.summary.quickstats.distributedCpuEntitlement}}).MHzReserved
$DRPComputeUsedTotal += $DRPComputeUsed
$DRPComputeReservedTotal += $DRPComputeReserved
}
$TotalDRPComputeUsed = ($DRPComputeUsedTotal | Measure-Object -Sum).Sum
$TotalDRPComputeReserved = ($DRPComputeReservedTotal | Measure-Object -Sum).Sum

#Get PAYG Pool Info
$PAYGs = Get-ResourcePool -location $selectedprp | Where-Object {($_.CPULimitMHz -eq "-1") -and ($_.MemLimitGB -eq "-1") -and ($_.Name -notmatch "Resource")}

$TotalPAYGComputeUsed = @()
ForEach ($PAYG in $PAYGs){
$PAYGComputeUsed = (Get-ResourcePool $PAYG | Get-View | Select @{Name="CPUUsage";Expression={$_.summary.quickstats.OverallCPuUsage}}).CPUUsage
$FRPComputeReserved = (Get-ResourcePool $PAYG | Get-View | Select @{Name="MHzReserved";Expression={$_.summary.quickstats.distributedCpuEntitlement}}).MHzReserved
$PAYGComputeUsedTotal += $PAYGComputeUsed
$PAYGComputeReservedTotal += $PAYGComputeReserved
}
$TotalPAYGComputeUsed = ($PAYGComputeUsedTotal | Measure-Object -Sum).Sum
$TotalPAYGComputeReserved = ($PAYGComputeReservedTotal | Measure-Object -Sum).Sum

#Calculate Percentages
$ClusterComputePercentUsed = [Math]::Round((($ClusterComputeUsed/$ClusterCompute)*100),2)
$TotalFRPComputePercentUsed = [Math]::Round((($TotalFRPComputeUsed/$ClusterCompute)*100),2)
$TotalDRPComputePercentUsed = [Math]::Round((($TotalDRPComputeUsed/$ClusterCompute)*100),2)
$TotalPAYGComputePercentUsed = [Math]::Round((($TotalPAYGComputeUsed/$ClusterCompute)*100),2)
#Combine Info
  $ComputeObj = new-object psobject -Property @{
				ClusterComputeUsed = $ClusterComputeUsed
				ClusterComputePercentUsed = $ClusterComputePercentUsed
				ClusterComputeTotal = $ClusterCompute
				TotalFRPComputeUsed = $TotalFRPComputeUsed
				TotalFRPComputeReserved = $TotalFRPComputeReserved
				TotalFRPComputePercentUsed = $TotalFRPComputePercentUsed
				TotalDRPComputeUsed = $TotalDRPComputeUsed
				TotalDRPComputeReserved = $TotalDRPComputeReserved
				TotalDRPComputePercentUsed = $TotalDRPComputePercentUsed
				TotalPAYGComputeUsed = $TotalPAYGComputeUsed
				TotalPAYGComputePercentUsed = $TotalPAYGComputePercentUsed
				TotalPAYGComputeReserved = $TotalPaygComputeReserved
								
				}
				
$clusterReturnedMetrics = "ClusterComputeUsed","ClusterComputePercentUsed","ClusterComputeTotal"
$FRPReturnedMetrics = "TotalFRPComputeUsed","TotalFRPComputeReserved","TotalFRPComputePercentUsed"
$DRPReturnedMetrics = "TotalDRPComputeUsed","TotalDRPComputeReserved","TotalDRPComputePercentUsed"
$PAYGReturnedMetrics = "TotalPAYGComputeUsed","TotalPAYGComputeReserved","TotalPAYGComputePercentUsed"

$computeReturned = $ComputeObj 

				write-output $computeReturned | Select $clusterReturnedMetrics | ft -autosize
				write-output $computeReturned | Select $FRPReturnedMetrics | ft -autosize
				write-output $computeReturned | Select $DRPReturnedMetrics | ft -autosize
				write-output $computeReturned | Select $PAYGReturnedMetrics | ft -autosize


# Get-resourcePool -norecursion -location (Get-Cluster CLUSTERNAME)