#Import required modules
Import-Module ImportExcel

#Capture process start time
$startTime = get-date 

#Define global variables
$date = get-date -format yyyy-MM-dd
$time = get-date -uformat %R
$filename = ".\CPU_Ready_" +"$date.csv"
$excelfilename = ".\VC02_CPU_Ready_" +"$date.xlsx"
$worksheetname = $date

#Begin collection of data
$VMs = (Get-VM | where {$_.PowerState -eq "PoweredOn"}).Name
#$VMs = 'TV-ESX-01', 'TV-ESX-02', 'TV-ESX-03'

$returnObj = @()
ForEach ($VMName in $VMs)
{       
	   $cpus = (Get-VM $VMName).NumCPU
	   $mem = (Get-VM $VMName).MemoryGB
	   $vmhost = (Get-VM $VMName).VmHost
	   $HostCPU = (Get-VMHost $vmhost).NumCPU
	   $hostusedmhz = (Get-VMHOst $vmhost).CpuUsageMhz
	   $hosttotalmhz = (Get-VMHOst $vmhost).CpuTotalMhz
	   $stats = get-vm $VMName | Get-Stat -realtime -maxsamples 1 -Stat Cpu.Ready.Summation -Instance ""
	   $aggr_stats = $stats | Measure-Object -Property Value -Average 
       $ready = [Math]::Round(((($aggr_stats.Average)/(20*1000) * 100)),2)
	   $rdy = [Math]::Round(($ready/$cpus),2)
	
	   $obj = new-object psobject -Property @{
				Date = $date
				Time = $time
				VMName = $VMName
				vCPUs = $cpus
				CPUReady = $rdy
				VMHost = $vmhost
				HostCPUCount = $hostcpu
				HostCPUUsedMHz = $hostusedmhz
				HostCPUTotalMHz = $hosttotalmhz 
				}
		$returnObj += $obj | Select Date,Time,VMName,vCPUs,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz | where {$_.CPUReady -ne "0"}
}

write-output $returnObj | Select Date,Time,VMName,vCPUs,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz |  Sort-Object -Property CPUReady -Descending | export-excel $excelfilename -worksheetname $worksheetname -startcolumn 2 -startrow 3 -append -Autosize 

#Calculate time required to execute script
$EndTime = get-date 
$Duration = $EndTime-$StartTime
$minutes = [Math]::Round(($DURATION.TotalMinutes),2) 
$seconds = [Math]::Round(($DURATION.TotalSeconds),1)
echo "$minutes minutes and $seconds seconds elapsed"