#Import required modules
Import-Module ImportExcel

#Capture process start time
$startTime = get-date 

#Define global variables
$date = get-date -format yyyy-MM-dd
$time = get-date -uformat %R
$filename = ".\CPU_Ready_" +"$date.csv"
$excelfilename = ".\VC02_CPU_Ready_" +"$date.xlsx"
$worksheetname = $date

#Define vCenters
$dtwdr = 'dtw-dr-vc01.dtw.vdc.local'
$dtwmgmt = 'dtw-mgmt-vc.dtw.vdc.local'
$kwddr = 'kwd-dr-vc01.dtw.vdc.local'
$kwdmgmt = 'kwd-mgmt-vc.dtw.vdc.local'
$vc01 = 'vmw-mgmt-vc01.grr.vdc.local'
$vc02 = 'vmw-mgmt-vc02.vdc.local'
$vc05 = 'vmw-mgmt-vc05.grr.vdc.local'
$vc06 = 'vmw-mgmt-vc06.dtw.vdc.local'
$vCs = $dtwdr,$dtwmgmt,$kwddr,$kwdmgmt,$vc01,$vc02,$vc05,$vc06

$countVC = 0
foreach($vC in $vCs){
Write-Output "[$countVC] $Vc"
$countVC = $countVC+1
}
$vcchoice = Read-Host "Which vCenter would you like to connect to?"
Write-Output " "
Write-host "please wait for script to finish, it may take some minutes." -ForegroundColor Yellow
connect-viserver $vcs[$vcchoice]

$Cluster = Get-Cluster
$countCL = 0
Write-Output " "
Write-Output "Clusters: "
Write-Output " "
foreach($oC in $Cluster){
Write-Output "[$countCL] $oc"
$countCL = $countCL+1
}
$choice = Read-Host "Which cluster would you like to review?"
Write-Output " "
Write-host "please wait for script to finish, it may take some minutes." -ForegroundColor Yellow
$cluster = get-cluster $cluster[$choice]
$vms = (Get-cluster $cluster | Get-VM | where {$_.PowerState -eq "PoweredOn"}).Name


$returnObj = @()
ForEach ($VMName in $VMs)
{      
	   $cpus = (Get-VM $VMName).NumCPU
	   $mem = (Get-VM $VMName).MemoryGB
	   $vmhost = (Get-VM $VMName).VmHost
	   $HostCPU = (Get-VMHost $vmhost).NumCPU
	   $hostusedmhz = (Get-VMHOst $vmhost).CpuUsageMhz
	   $hosttotalmhz = (Get-VMHOst $vmhost).CpuTotalMhz
	   $stats = get-vm $VMName | where {$_.PowerState -eq "PoweredOn"} | Get-Stat -realtime -maxsamples 1 -Stat Cpu.Ready.Summation -Instance ""
	   $aggr_stats = $stats | Measure-Object -Property Value -Average 
       $ready = [Math]::Round(((($aggr_stats.Average)/(20*1000) * 100)),2)
	   $rdy = [Math]::Round(($ready/$cpus),2)
	
	   $obj = new-object psobject -Property @{
				Date = $date
				Time = $time
				VMName = $VMName
				vCPUs = $cpus
				CPUReady = $rdy
				VMHost = $vmhost
				HostCPUCount = $hostcpu
				HostCPUUsedMHz = $hostusedmhz
				HostCPUTotalMHz = $hosttotalmhz 
				}
		$returnObj += $obj | Select Date,Time,VMName,vCPUs,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz | where {$_.CPUReady -ne "0"}
}

write-output $returnObj | Select Date,Time,VMName,vCPUs,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz |  Sort-Object -Property CPUReady -Descending | export-excel $excelfilename -worksheetname $worksheetname -startcolumn 2 -startrow 3 -append -Autosize 

#Calculate time required to execute script
$EndTime = get-date 
$Duration = $EndTime-$StartTime
$minutes = [Math]::Round(($DURATION.TotalMinutes),2) 
$seconds = [Math]::Round(($DURATION.TotalSeconds),1)
echo "$minutes minutes and $seconds seconds elapsed"