#Import required modules
Import-Module ImportExcel

#Capture process start time
$startTime = get-date 

#Define global variables
$date = get-date -format yyyy-MM-dd
$time = get-date -uformat %R

#Define vCenters
$dtwdr = 'dtw-dr-vc01.dtw.vdc.local'
$dtwmgmt = 'dtw-mgmt-vc.dtw.vdc.local'
$kwddr = 'kwd-dr-vc01.kwd.vdc.local'
$kwdmgmt = 'kwd-mgmt-vc.kwd.vdc.local'
$vc01 = 'vmw-mgmt-vc01.grr.vdc.local'
$vc02 = 'vmw-mgmt-vc02.vdc.local'
$vc05 = 'vmw-mgmt-vc05.grr.vdc.local'
$vc06 = 'vmw-mgmt-vc06.dtw.vdc.local'
$vCenters = $dtwdr,$dtwmgmt,$kwddr,$kwdmgmt,$vc01,$vc02,$vc05,$vc06

#Clear screen to reduce clutter
cls

#Select which vCenter to connect to
$countVC = 0
Write-Output " "
foreach($vC in $vCenters){
Write-Output "[$countVC] $vC"
$countVC = $countVC+1
}
write-output " "
$choice = Read-Host "Which vCenter would you like to connect to?"
Write-Output " "
#Connect to the Desired vCenter
$connection = connect-viserver $vCenters[$choice]
Write-output "Connection established"
write-output " "
$choice = 0

#Clear screen to reduce clutter
cls

Write-Output " "
$RPinput = Read-Host "Please enter circuit ID"
$RPlist = Get-ResourcePool | Where-Object {($_.Name -match $RPinput)}
#Select specific resource pool
$countRP = 0
Write-Output " "
foreach($RP in $RPlist){
Write-Output "[$countRP] $RP"
$countRP = $countRP+1
}
write-output " "
$rpchoice = Read-Host "Please select the resource pool you would like to review"
Write-Output " "
$resourcepool = Get-ResourcePool $rplist[$rpchoice]
$choice = 0

#Collect and organize resource pool data
$rplimits = Get-ResourcePool $resourcepool |select Name,CpuReservationMHz,CpuLimitMHz,MemReservationGB,MemLimitGB
$rpquickstats = 


#Select only the powered-on VMs from the desired resource pool
$vms = (Get-ResourcePool $resourcepool | Get-VM | where {$_.PowerState -eq "PoweredOn"}).Name

#Collect and organize VM data
$returnObj = @()
ForEach ($VM in $VMs){           
		$vmquickstats = get-vm $VM | get-view | Select @{Name="Swapped";Expression={$_.summary.quickstats.swappedmemory}},@{Name="Ballooned";Expression={$_.summary.quickstats.balloonedmemory}},@{Name="CPUUsage";Expression={$_.summary.quickstats.overallCpuUsage}},@{Name="MemUsage";Expression={$_.summary.quickstats.hostMemoryUsage}},@{Name="CpuDemand";Expression={$_.summary.quickstats.overallCpuDemand}},@{Name="CpuEntitlement";Expression={$_.summary.quickstats.staticCpuEntitlement}},@{Name="MemoryEntitlement";Expression={$_.summary.quickstats.staticMemoryEntitlement}}

	   $swap = $vmquickstats.Swapped
       $balloon = $vmquickstats.Ballooned
	   $cpuusage = $vmquickstats.CPUUsage
	   $memusage = $vmquickstats.MemUsage
	   $cpudemand = $vmquickstats.CPUDemand
   	   $cpuentitlement = $vmquickstats.CPUEntitlement	   
	   $mementitlement = $vmquickstats.MemoryEntitlement
	   
	   $vminfo = Get-VM $VM 
	   $cpus = $vminfo.NumCPU
	   $mem = $vminfo.MemoryGB
	   $vmlimit = (Get-VM $vminfo | Get-VMResourceConfiguration).CPULimitMHz
	   $stats = get-vm $VM | Get-Stat -realtime -maxsamples 300 -Stat Cpu.Ready.Summation -Instance ""
	   $aggr_stats = $stats | Measure-Object -Property Value -Average 
       $ready = [Math]::Round(((($aggr_stats.Average)/(20*1000) * 100)),2)
	   $rdy = [Math]::Round(($ready/$cpus),2)
	   $csvfilename = ".\$ResourcePool" + "$date.csv"
	   $obj = new-object psobject -Property @{
				Date = $date
				Time = $time
				VMName = $VM
				vCPUs = $cpus
				cpuusage = $cpuusage
				memusage = $memusage
				vMem = $mem
				CPUReady = $rdy
				Swap = $swap
				Balloon = $balloon
				VMLimit = $vmlimit
				cpudemand = $cpudemand			
				cpuentitlement = $cpuentitlement
				memoryentitlement = $mementitlement	
				}
				$returnedMetrics = "Date","Time","VMName","vCPUs","cpuusage","vMem","memusage","Swap","Balloon","VMLimit","CPUReady","CPUDemand","CPUEntitlement","MemoryEntitlement"
		$returnObj += $obj | Select  $returnedMetrics #Date,Time,VMName,vCPUs,cpuusage,vMem,memusage,Swap,Balloon,VMLimit,CPUReady #| where ({$_.CPUReady -ne "0"})
}
#Paramters for sorting returned values
$prop1 = @{Expression="CPUReady";Descending=$true}
$prop2 = @{Expression="VMLimit";Ascending=$true}
$prop3 = @{Expression="Swap";Descending=$true}

#Return collected VM data to screen and to file
$output = $returnObj
write-output $output | Select $returnedMetrics |  Sort-Object -Property $prop3, $prop2, $prop1 | ft -autosize 
write-output $output | Select $returnedMetrics |  Sort-Object -Property $prop3, $prop2, $prop1 | export-csv -Path $csvfilename -notypeinformation



#Calculate time required to execute script
$EndTime = get-date 
$Duration = $EndTime-$StartTime
echo " "
echo "Operation took $duration to complete"
#Disconnect from all vCenters
Disconnect-VIServer -Server * -Force -Confirm:$false

