#Import required modules
#Import-Module ImportExcel

#Capture process start time
$startTime = get-date 

#Define global variables
$date = get-date -format yyyy-MM-dd
$time = get-date -uformat %R

#Define vCenters
$dtwdr = 'dtw-dr-vc01.dtw.vdc.local'
$dtwmgmt = 'dtw-mgmt-vc.dtw.vdc.local'
$kwddr = 'kwd-dr-vc01.kwd.vdc.local'
$kwdmgmt = 'kwd-mgmt-vc.kwd.vdc.local'
$vc01 = 'vmw-mgmt-vc01.grr.vdc.local'
$vc02 = 'vmw-mgmt-vc02.vdc.local'
$vc05 = 'vmw-mgmt-vc05.grr.vdc.local'
$vc06 = 'vmw-mgmt-vc06.dtw.vdc.local'
$vCenters = $dtwdr,$dtwmgmt,$kwddr,$kwdmgmt,$vc01,$vc02,$vc05,$vc06

cls

#Select which vCenter to connect to
$countVC = 0
Write-Output " "
foreach($vC in $vCenters){
Write-Output "[$countVC] $vC"
$countVC = $countVC+1
}
write-output " "
$choice = Read-Host "Which vCenter would you like to connect to?"
Write-Output " "
#Connect to the Desired vCenter
$connection = connect-viserver $vCenters[$choice]
Write-output "Connection established"
write-output " "
$choice = 0
cls



Write-Output " "
$RPinput = Read-Host "Please enter circuit ID"
$RPlist = Get-ResourcePool | Where-Object {($_.Name -match $RPinput)}
#Select specific resource pool
$countRP = 0
Write-Output " "
foreach($RP in $RPlist){
Write-Output "[$countRP] $RP"
$countRP = $countRP+1
}
write-output " "
$rpchoice = Read-Host "Please select the resource pool you would like to review"
Write-Output " "
$resourcepool = Get-ResourcePool $rplist[$rpchoice]
$choice = 0


$vms = (Get-ResourcePool $resourcepool | Get-VM | where {$_.PowerState -eq "PoweredOn"}).Name
# $vms
# $vmstats = get-vm $vms | Get-Stat -realtime -maxsamples 1 -Stat Cpu.Ready.Summation -Instance "" | Select Value,Entity

#Collect and organize data
$returnObj = @()
ForEach ($VM in $VMs)
{       
	   $vminfo = Get-VM $VM 
	   $cpus = $vminfo.NumCPU
	   $mem = $vminfo.MemoryGB
	   $vmlimit = (Get-VM $vminfo | Get-VMResourceConfiguration).CPULimitMHz
	   $stats = get-vm $VM | Get-Stat -realtime -maxsamples 300 -Stat Cpu.Ready.Summation -Instance ""
	   $aggr_stats = $stats | Measure-Object -Property Value -Average 
       $ready = [Math]::Round(((($aggr_stats.Average)/(20*1000) * 100)),2)
	   $rdy = [Math]::Round(($ready/$cpus),2)
	   #$worksheetname = $vmhost
	   #$excelfilename = ".\$vmhost" + "_CPU_Ready_" +"$date.xlsx"
	   $obj = new-object psobject -Property @{
				Date = $date
				Time = $time
				VMName = $VM
				vCPUs = $cpus
				vMem = $mem
				CPUReady = $rdy
				VMLimit = $vmlimit
				}
		$returnObj += $obj | Select Date,Time,VMName,vCPUs,vMem,VMLimit,CPUReady #| where ({$_.CPUReady -ne "0"})
}
#Paramters for sorting returned values
$prop1 = @{Expression="CPUReady";Descending=$true}
$prop2 = @{Expression="VMLimit";Ascending=$true}

#Return collected data
$output = $returnObj
# $returnObj

#write-output $output | Select Date,Time,VMName,vCPUs,VMLimit,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz |  Sort-Object -Property $prop2, $prop1 | export-excel $excelfilename -worksheetname $worksheetname -startcolumn 2 -startrow 3 -append -Autosize
write-output $output | Select Date,Time,VMName,vCPUs,vMem,VMLimit,CPUReady |  Sort-Object -Property $prop2, $prop1 | ft -autosize 

#Calculate time required to execute script
$EndTime = get-date 
$Duration = $EndTime-$StartTime
echo " "
echo "Operation took $duration to complete"
#Disconnect from all vCenters
Disconnect-VIServer -Server * -Force -Confirm:$false

