#Import required modules
Import-Module ImportExcel

#Capture process start time
$startTime = get-date 

#Define global variables
$date = get-date -format yyyy-MM-dd
$time = get-date -uformat %R
$filename = ".\CPU_Ready_" +"$date.csv"
#$excelfilename = ".\CPU_Ready_" +"$date.xlsx"
#$worksheetname = $date

#Define vCenters
$dtwdr = 'dtw-dr-vc01.dtw.vdc.local'
$dtwmgmt = 'dtw-mgmt-vc.dtw.vdc.local'
$kwddr = 'kwd-dr-vc01.kwd.vdc.local'
$kwdmgmt = 'kwd-mgmt-vc.kwd.vdc.local'
$vc01 = 'vmw-mgmt-vc01.grr.vdc.local'
$vc02 = 'vmw-mgmt-vc02.vdc.local'
$vc05 = 'vmw-mgmt-psc05.grr.vdc.local'
$vc06 = 'vmw-mgmt-vc06.dtw.vdc.local'
$vCenters = $dtwdr,$dtwmgmt,$kwddr,$kwdmgmt,$vc01,$vc02,$vc05,$vc06



#Select which vCenter to connect to
$countVC = 0
Write-Output " "
foreach($vC in $vCenters){
Write-Output "[$countVC] $vC"
$countVC = $countVC+1
}
write-output " "
$choice = Read-Host "Which vCenter would you like to connect to?"
Write-Output " "
#Connect to the Desired vCenter
$connection = connect-viserver $vCenters[$choice]
Write-output "Connection established"
write-output " "
$choice = 0

#Select the cluster for data gathering
$Cluster = (Get-Cluster | Sort-Object -Property Name).Name
$countCL = 0
Write-Output " "
Write-Output "Clusters: "
Write-Output " "
foreach($oC in $Cluster){
Write-Output "[$countCL] $oc"
$countCL = $countCL+1
}
Write-Output " "
$choice = Read-Host "Which cluster would you like to review?"
Write-Output " "
# Write-host "please wait for script to finish, it may take some minutes." -ForegroundColor Yellow
$cluster = get-cluster $cluster[$choice]
$choice = 0


#Select the ESXi Host for data gathering
$esxhosts = (Get-Cluster $cluster | Get-VMHost | Sort-Object -Property Name).Name
$countEsx = 0
Write-Output " "
Write-Output "Hosts: "
Write-Output " "
foreach($ESXhost in $ESXhosts){
Write-Output "[$countEsx] $ESXHost"
$countEsx = $countEsx+1
}
Write-Output " "
$choice = Read-Host "Which ESX host would you like to review?"
Write-Output " "
Write-host "please wait for script to finish, it may take some minutes." -ForegroundColor Yellow
$esxihost = get-vmhost $esxhosts[$choice]
$choice = 0

$vms = (Get-VMHost $esxihost  | Get-VM | where {$_.PowerState -eq "PoweredOn"}).Name
$vmstats = get-vm $vms | Get-Stat -realtime -maxsamples 1 -Stat Cpu.Ready.Summation -Instance "" | Select Value,Entity

#Collect and organize data
$returnObj = @()
ForEach ($VMName in $VMs)
{       
	   $vminfo = Get-VM $VMName
	   $cpus = $vminfo.NumCPU
	   $mem = $vminfo.MemoryGB
	   $vmlimit = (Get-VM $vminfo | Get-VMResourceConfiguration).CPULimitMHz
	   $vmhost = $vminfo.VmHost
	   $hostinfo = (Get-VMHost $vmhost)
	   $HostCPU = $hostinfo.NumCPU
	   $hostusedmhz = $hostinfo.CpuUsageMhz
	   $hosttotalmhz = $hostinfo.CpuTotalMhz
	   $stats = get-vm $VMName | Get-Stat -realtime -maxsamples 300 -Stat Cpu.Ready.Summation -Instance ""
	   $aggr_stats = $stats | Measure-Object -Property Value -Average 
       $ready = [Math]::Round(((($aggr_stats.Average)/(20*1000) * 100)),2)
	   $rdy = [Math]::Round(($ready/$cpus),2)
	   $worksheetname = $vmhost
	   $excelfilename = ".\$vmhost" + "_CPU_Ready_" +"$date.xlsx"
	   $obj = new-object psobject -Property @{
				Date = $date
				Time = $time
				VMName = $VMName
				vCPUs = $cpus
				CPUReady = $rdy
				VMLimit = $vmlimit
				VMHost = $vmhost
				HostCPUCount = $hostcpu
				HostCPUUsedMHz = $hostusedmhz
				HostCPUTotalMHz = $hosttotalmhz
				}
		$returnObj += $obj | Select Date,Time,VMName,vCPUs,VMLimit,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz #| where ({$_.CPUReady -ne "0"})
}
#Paramters for sorting returned values
$prop1 = @{Expression="CPUReady";Descending=$true}
$prop2 = @{Expression="VMLimit";Ascending=$true}

#Return collected data
$output = $returnObj
write-output $output | Select Date,Time,VMName,vCPUs,VMLimit,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz |  Sort-Object -Property $prop2, $prop1 | export-excel $excelfilename -worksheetname $worksheetname -startcolumn 2 -startrow 3 -append -Autosize
write-output $output | Select Date,Time,VMName,vCPUs,VMLimit,CPUReady,VMHost,HostCPUCount,HostCPUUsedMHz,HostCPUTotalMHz |  Sort-Object -Property $prop2, $prop1 | ft -autosize 


#Calculate time required to execute script
$EndTime = get-date 
$Duration = $EndTime-$StartTime
echo "$duration elapsed"
echo " "
#Disconnect from all vCenters
Disconnect-VIServer -Server * -Force -Confirm:$false
