#Begin
Import-Module ImportExcel

#UCS Power Stats
#Get-UcsBlade | Get-UcsComputeBoard | Get-UcsComputeMbPowerStats

#Variables
	#UCS Domains
		$kwd1 = "UCS-gdrsmiuk.kwd.vdc.local"			#172.19.1.20	
		$kwd2 = "UCS-gdrsmiuk-02.kwd.vdc.local"			#172.19.6.20	
		$kwd3 = "UCS-gdrsmiuk-03.kwd.vdc.local"			#172.19.6.10	
		$dtw1 = "UCS-MLRSMI00.dtw.vdc.local"			#172.16.50.60
		$dtw2 = "UCS-MLRSMI00-02.dtw.vdc.local"			#172.16.54.60
		$dtw3 = "UCS-MLRSMI00-03.dtw.vdc.local"			#172.16.54.70
	
		$UcsDomains = $kwd1,$kwd2,$kwd3,$dtw1,$dtw2,$dtw3
		
	#Credentials
	$c = Get-Credential
	$date = get-date -format yyyy-MM-dd

	
ForEach ($UCSDomain in $UCSDomains)
{
#Connect and Collect
Connect-Ucs $UCSDomain -Credential $c
$filename = ".\" + $UCSDomain + "_" +"$date.xlsx"
$UCSchassis = (get-ucschassis).ID

#Overall Slot Population Report
	#Collect Slot Information
	Get-UcsFabricComputeSlotEp | Select ChassisID,Rn,Presence,Model,Serial | export-excel $filename -worksheetname "Slot Population" -startcolumn 2 -startrow 2 -AutoSize

	#Set up Slot Population Worksheet - Overwrite default headers from collection command above
	echo "Chassis ID" | export-excel $filename -worksheetname "Slot Population" -startcolumn 2 -startrow 2 -AutoSize
	echo "Slot #" | export-excel $filename -worksheetname "Slot Population" -startcolumn 3 -startrow 2 -AutoSize
	echo "Status" | export-excel $filename -worksheetname "Slot Population" -startcolumn 4 -startrow 2 -AutoSize
	echo "Blade Model" | export-excel $filename -worksheetname "Slot Population" -startcolumn 5 -startrow 2 -AutoSize
	echo "Serial Number" | export-excel $filename -worksheetname "Slot Population" -startcolumn 6 -startrow 2 -AutoSize

#Empty Slot Report
	#Collect EmptySlot Information
	Get-UcsFabricComputeSlotEp | Where-Object {($_.Presence -eq 'empty')} | Select ChassisID,Rn,Presence | export-excel $filename -worksheetname "Empty Slots" -startcolumn 2 -startrow 2 -AutoSize

	#Set up Slot Population Worksheet - Overwrite default headers from collection command above
	echo "Chassis ID" | export-excel $filename -worksheetname "Empty Slots" -startcolumn 2 -startrow 2 -AutoSize
	echo "Slot #" | export-excel $filename -worksheetname "Empty Slots" -startcolumn 3 -startrow 2 -AutoSize
	echo "Status" | export-excel $filename -worksheetname "Empty Slots" -startcolumn 4 -startrow 2 -AutoSize

#Collect Stats for Each Chassis
		ForEach ($UCSChassisID in $UCSchassis)
		{
		#Set up output file
		echo "Chassis #" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 1 -startrow 1 -AutoSize
		echo $UCSchassisID | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 2 -startrow 1  -AutoSize
		#odd slots
		echo "Profile Assigned" | export-excel $filename -worksheetname $UCSchassisID -startcolumn 2 -startrow 2
		echo "Slot 1" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 1 -startrow 3 -AutoSize
		echo "Slot 3" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 1 -startrow 4 -AutoSize
		echo "Slot 5" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 1 -startrow 5 -AutoSize
		echo "Slot 7" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 1 -startrow 6 -AutoSize
		#even slots
		echo "Profile Assigned" | export-excel $filename -worksheetname $UCSchassisID -startcolumn 5 -startrow 2 -AutoSize
		echo "Slot 2" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 4 -startrow 3 -AutoSize
		echo "Slot 4" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 4 -startrow 4 -AutoSize
		echo "Slot 6" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 4 -startrow 5 -AutoSize
		echo "Slot 8" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 4 -startrow 6 -AutoSize
		
		#Set up columns for individual blade information
		echo "Blade" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 1 -startrow 8 -AutoSize
		echo "Assigned Profile" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 2 -startrow 8 -AutoSize
		echo "User Label" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 3 -startrow 8 -AutoSize
		echo "Model" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 4 -startrow 8 -AutoSize
		echo "VID" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 5 -startrow 8 -AutoSize
		echo "Serial #" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 6 -startrow 8 -AutoSize
		echo "Association Status" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 7 -startrow 8 -AutoSize
		echo "Total RAM" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 8 -startrow 8 -AutoSize
		echo "CPUs Installed" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 9 -startrow 8 -AutoSize
		echo "CPU Cores" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 10 -startrow 8 -AutoSize
		echo "UCS Domain" | export-excel $filename -worksheetname $UCSchassisID  -startcolumn 11 -startrow 8 -AutoSize
				
		$OddBlade = "Blank"
		$OddBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 1)}).AssignedTodn
		echo $OddBlade | export-excel $filename -worksheetname $UCSchassisID -startcolumn 2 -startrow 3 -Autosize
		$OddBlade = "Blank"
		$OddBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 3)}).AssignedTodn
		echo $OddBlade | export-excel $filename -worksheetname $UCSchassisID -startcolumn 2 -startrow 4 -Autosize
		$OddBlade = "Blank"
		$OddBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 5)}).AssignedTodn
		echo $OddBlade | export-excel $filename -worksheetname $UCSchassisID -startcolumn 2 -startrow 5 -Autosize
		$OddBlade = "Blank"
		$OddBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 7)}).AssignedTodn
		echo $OddBlade | export-excel $filename -worksheetname $UCSchassisID -startcolumn 2 -startrow 6 -Autosize
		
		
		$EvenBlade = "Blank"
		$EvenBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 2)}).AssignedTodn
		echo $EvenBlade | export-excel $filename -worksheetname $UCSchassisID -startcolumn 5 -startrow 3 -Autosize
		$EvenBlade = "Blank"
		$EvenBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 4)}).AssignedTodn
		echo $EvenBlade | export-excel $filename -worksheetname $UCSchassisID -startcolumn 5 -startrow 4 -Autosize
		$EvenBlade = "Blank"
		$EvenBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 6)}).AssignedTodn
		echo $EvenBlade | export-excel $filename -worksheetname $UCSchassisID -startcolumn 5 -startrow 5 -Autosize
		$EvenBlade = "Blank"
		$EvenBlade = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSchassisID) -and ($_.SlotID -eq 8)}).AssignedTodn
		echo $EvenBlade | export-excel $filename -worksheetname $UCSchassisID -startcolumn 5 -startrow 6 -Autosize
	
	
			
		#Grab Blade Information
		$BladeSlotID = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).SlotID
		echo $BladeSlotID | export-excel $filename -worksheetname $UCSchassisID -startcolumn 1 -startrow 9 -AutoSize
		
		$BladeAssignedToDn = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).AssignedToDn 
		echo $BladeAssignedToDn | export-excel $filename -worksheetname $UCSchassisID -startcolumn 2 -startrow 9 -AutoSize
		
		$BladeUsrLbl = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).UsrLbl 
		echo $BladeUsrLbl | export-excel $filename -worksheetname $UCSchassisID -startcolumn 3 -startrow 9 -AutoSize
		
		$BladeModel = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).Model 
		echo $BladeModel | export-excel $filename -worksheetname $UCSchassisID -startcolumn 4 -startrow 9 -AutoSize
		
		$BladeVID = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).VID 
		echo $BladeVID | export-excel $filename -worksheetname $UCSchassisID -startcolumn 5 -startrow 9	-AutoSize
		
		$BladeSerial = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).Serial 
		echo $BladeSerial | export-excel $filename -worksheetname $UCSchassisID -startcolumn 6 -startrow 9 -AutoSize
		
		$BladeAssociation = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).Association
		echo $BladeAssociation | export-excel $filename -worksheetname $UCSchassisID -startcolumn 7 -startrow 9 -AutoSize
		
		$BladeTotalMemory = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).TotalMemory
		echo $BladeTotalMemory | export-excel $filename -worksheetname $UCSchassisID -startcolumn 8 -startrow 9 -AutoSize
		
		$BladeNumOfCpus = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).NumOfCpus
		echo $BladeNumOfCpus | export-excel $filename -worksheetname $UCSchassisID -startcolumn 9 -startrow 9 -AutoSize
		
		$BladeNumOfCores = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).NumOfCores
		echo $BladeNumOfCores | export-excel $filename -worksheetname $UCSchassisID -startcolumn 10 -startrow 9 -AutoSize
		 
		$BladeMkt = (get-ucschassis | Get-UcsBlade | Where-Object {($_.ChassisID -eq $UCSChassisID) } ).Ucs
		echo $BladeMkt | export-excel $filename -worksheetname $UCSchassisID -startcolumn 11 -startrow 9 -AutoSize
		
		}
Disconnect-Ucs
}
#end