﻿#Select only the powered-on VMs from the desired resource pool
$vms = (Get-ResourcePool $resourcepool | Get-VM | where {$_.PowerState -eq "PoweredOn"}).Name

#Collect and organize VM data
$returnObj = @()
ForEach ($VM in $VMs){           
		$vmquickstats = get-vm $VM | get-view | Select @{Name="Swapped";Expression={$_.summary.quickstats.swappedmemory}},@{Name="Ballooned";Expression={$_.summary.quickstats.balloonedmemory}},@{Name="CPUUsage";Expression={$_.summary.quickstats.overallCpuUsage}},@{Name="MemUsage";Expression={$_.summary.quickstats.hostMemoryUsage}},@{Name="CpuDemand";Expression={$_.summary.quickstats.overallCpuDemand}},@{Name="CpuEntitlement";Expression={$_.summary.quickstats.staticCpuEntitlement}},@{Name="MemoryEntitlement";Expression={$_.summary.quickstats.staticMemoryEntitlement}}

	   $swap = $vmquickstats.Swapped
       $balloon = $vmquickstats.Ballooned
	   $cpuusage = $vmquickstats.CPUUsage
	   $memusage = $vmquickstats.MemUsage
	   $cpudemand = $vmquickstats.CPUDemand
   	   $cpuentitlement = $vmquickstats.CPUEntitlement	   
	   $mementitlement = $vmquickstats.MemoryEntitlement
	   
	   $vminfo = Get-VM $VM 
	   $cpus = $vminfo.NumCPU
	   $mem = $vminfo.MemoryGB
	   $vmlimit = (Get-VM $vminfo | Get-VMResourceConfiguration).CPULimitMHz
	   $stats = get-vm $VM | Get-Stat -realtime -maxsamples 300 -Stat Cpu.Ready.Summation -Instance ""
	   $aggr_stats = $stats | Measure-Object -Property Value -Average 
       $ready = [Math]::Round(((($aggr_stats.Average)/(20*1000) * 100)),2)
	   $rdy = [Math]::Round(($ready/$cpus),2)
	   $csvfilename = ".\$ResourcePool" + "$date.csv"
	   $obj = new-object psobject -Property @{
				Date = $date
				Time = $time
				VMName = $VM
				vCPUs = $cpus
				cpuusage = $cpuusage
				memusage = $memusage
				vMem = $mem
				CPUReady = $rdy
				Swap = $swap
				Balloon = $balloon
				VMLimit = $vmlimit
				cpudemand = $cpudemand			
				cpuentitlement = $cpuentitlement
				memoryentitlement = $mementitlement	
				}
				$returnedMetrics = "Date","Time","VMName","vCPUs","cpuusage","vMem","memusage","Swap","Balloon","VMLimit","CPUReady","CPUDemand","CPUEntitlement","MemoryEntitlement"
		$returnObj += $obj | Select  $returnedMetrics #Date,Time,VMName,vCPUs,cpuusage,vMem,memusage,Swap,Balloon,VMLimit,CPUReady #| where ({$_.CPUReady -ne "0"})